dnl aclocal.m4 - Autoconf macros used at BBL



dnl Report compiler options selected by AC_PROC_CC.
dnl
dnl This is mostly for debugging the following macros, by making
dnl it clear who set particular options.  It should be used after
dnl AC_PROG_CC but before other macros to set compiler options.

AC_DEFUN(BBL_MSG_C_OPTIONS,
[AC_MSG_CHECKING(C compiler basic options)
AC_MSG_RESULT($CFLAGS)
])



dnl Choose C compiler options to select C99 dialect.
dnl 
dnl Appends the needed flags to CFLAGS.  Currently works only for gcc.  
dnl NOTE that choosing gnu99 rather than c99 brings in more than strict 
dnl C99 compliance and might make it harder to port later; but there are
dnl already some non-standard things (getopt, strdup) which Pyvox expects to 
dnl have that we don't get with the -std=c99 option.

AC_DEFUN(BBL_C99_FLAGS,
[AC_MSG_CHECKING(C compiler flags for C99 dialect)
if test "$GCC" = "yes"; then
   CFLAGS="$CFLAGS -std=gnu99"
fi
AC_MSG_RESULT($CFLAGS)
])



dnl Choose C compiler flags to get additional warnings.
dnl
dnl Defines CWARNINGS as the needed flags and substitutes.
dnl Currently works well only for gcc,
dnl since it must know explicitly what flags to use for which compilers.
dnl The particular flags chosen for gcc are those used within Pyvox
dnl and are chosen to maximize the number of dubious constructs used
dnl without yielding too many false positives; other programmers may 
dnl disagree.
dnl
dnl Adding "-Wpointer-arith" for gcc would be useful for catching some
dnl portability errors, but also causes many spurious warnings for strcpy 
dnl and memset as implemented in RedHat's glibc-2.2.2-10.src.rpm.
dnl
dnl Adding "-ansi" for gcc catches some potential portability problems 
dnl but also generates spurious warnings for the common functions
dnl snprintf, mkstemp, and strdup.  Adding "-pedantic" seems to generate
dnl all sorts of useless warnings and no useful ones.

AC_DEFUN(BBL_C_WARNINGS,
[AC_MSG_CHECKING(C compiler flags for warnings)
if test "$GCC" = "yes"; then
   CWARNINGS="-Wall -Wmissing-prototypes -Wstrict-prototypes"
fi
AC_SUBST(CWARNINGS)
AC_MSG_RESULT($CWARNINGS)
])



dnl Check flag(s) to generate position-independent code.
dnl 
dnl Set PICFLAG to the right value and substitute where needed.
dnl
dnl FIXME: Currently works only for gcc

AC_DEFUN(BBL_C_PIC,
[AC_MSG_CHECKING(C compiler flags for position-independent code)
if test "$OS" = "Darwin" -a "$GCC" = "yes"; then
   PICFLAG=" "
   AC_MSG_RESULT(none needed)
elif test "$GCC" = "yes"; then
   PICFLAG="-fPIC"
   AC_MSG_RESULT($PICFLAG)
fi
AC_SUBST(PICFLAG)
])



dnl Check flag(s) to link object files into a shared library.
dnl 
dnl Set SHARED to the right value and substitute where needed.
dnl
dnl FIXME: Currently works only for gcc

AC_DEFUN(BBL_C_SHARED,
[AC_MSG_CHECKING(C compiler flags to link a shared library)
if test "$OS" = "Darwin" -a "$GCC" = "yes"; then
   SHARED="-dynamiclib"
elif test "$GCC" = "yes"; then
   SHARED="-shared"
fi
AC_MSG_RESULT($SHARED)
AC_SUBST(SHARED)
])



dnl Check C compiler extension for a shared library.
dnl
dnl Set SHEXT to the right value and substitute where needed.
dnl
dnl FIXME: Currently works only for gcc

AC_DEFUN(BBL_C_SHARED_EXT,
[AC_MSG_CHECKING(C compiler extension for a shared library)
if test "$OS" = "Darwin" -a "$GCC" = "yes"; then
   SHEXT="dylib"
elif test "$GCC" = "yes"; then
   SHEXT="so"
fi
AC_MSG_RESULT($SHEXT)
AC_SUBST(SHEXT)
])



dnl Check flag(s) to link binaries into a dynamically loaded module.
dnl 
dnl Set BUNDLE to the right value, including the flags needed to
dnl suppress complaints about missing symbols to be provided by the
dnl host program, and substitute where needed.
dnl
dnl FIXME: Currently works only for gcc

AC_DEFUN(BBL_C_BUNDLE,
[AC_MSG_CHECKING(C compiler flags for a dynamically loaded module)
if test "$OS" = "Darwin" -a "$GCC" = "yes"; then
   BUNDLE="-bundle"
elif test "$GCC" = "yes"; then
   BUNDLE="-shared"
fi
AC_MSG_RESULT($BUNDLE)
AC_SUBST(BUNDLE)
])



dnl Check flag(s) to compile/link as a plugin.
dnl 
dnl Set PLUGIN to the right value and substitute where needed.
dnl The primary function of these flags is to suppress warnings/errors
dnl for undefined symbols that will be defined by the host application;
dnl but note that this can also conceal errors.
dnl
dnl FIXME: Currently works only for gcc

AC_DEFUN(BBL_C_PLUGIN,
[AC_MSG_CHECKING(C compiler flags to link as a plugin)
if test "$OS" = "Darwin" -a "$GCC" = "yes"; then
   PLUGIN="-undefined suppress -flat_namespace"
   AC_MSG_RESULT($PLUGIN)
elif test "$GCC" = "yes"; then
   PLUGIN=""
   AC_MSG_RESULT(none needed)
fi
AC_SUBST(PLUGIN)
])



dnl Check if the C compiler supports the restrict keyword, or an equivalent.
dnl
dnl If the compiler accepts "restrict", do nothing; if is accepts
dnl "__restrict" or "__restrict__", use that instead; otherwise define
dnl restrict to be empty.

AC_DEFUN(BBL_C_RESTRICT,
[AC_CACHE_CHECK([for restrict], bbl_cv_c_restrict,
[bbl_cv_c_restrict=no
for bbl_kw in restrict __restrict __restrict__ ; do
  AC_TRY_COMPILE(, [int * $bbl_kw p;], [bbl_cv_c_restrict=$bbl_kw; break])
done
])
case "$bbl_cv_c_restrict" in
  restrict | yes) ;;
  no) AC_DEFINE(restrict, ) ;;
  *)  AC_DEFINE_UNQUOTED(restrict, $bbl_cv_c_restrict) ;;
esac
])



dnl BBL_C_INLINE - Check for the inline keyword or equivalent.
dnl 
dnl This macro sets appropriate flags depending on whether or not
dnl the compiler accepts the inline keyword or some equivalent.
dnl The C macro "inline" is undefined if the compiler accepts "inline",
dnl to an equivalent accepted by the compiler, or to blank.  The C
dnl macro HAVE_INLINE is defined as 1 if the compiler accepts inline
dnl or some equivalent, and undefined otherwise.  The C macro inlineable
dnl is set to "static inline" if the compiler accepts inline or some
dnl equivalent, and to "extern" otherwise.

AC_DEFUN(BBL_C_INLINE,
[AC_CACHE_CHECK([for working inline], bbl_c_inline,
[bbl_c_inline=no
for kw in inline __inline__ __inline; do
  AC_TRY_COMPILE(, [} static $kw foo() {], [bbl_c_inline=$kw; break])
done
])
case "$bbl_c_inline" in
  inline | yes) 
    AC_DEFINE(HAVE_INLINE, 1)
    AC_DEFINE(inlineable, static inline)
    ;;
  no) 
    AC_DEFINE(inline, )
    AC_DEFINE(inlineable, extern)
    ;;
  *)
    AC_DEFINE_UNQUOTED(inline, $bbl_c_inline)
    AC_DEFINE(HAVE_INLINE, 1)
    AC_DEFINE(inlineable, static inline)
    ;;
esac
])
