/****************************************************************************
vxli.h - Private header file for the Voxel Kit

Author:  Paul Hughett

Copyright (c) 1998-2003 Trustees of the University of Pennsylvania
See the file COPYING for license terms.

This file defines private structures and prototypes for the Voxel Kit.
If the compiler supports inlining, then it also includes the definitions
of the inlineable internal functions.

FIXME:  Check that all the public and private functions are correctly
classified.

FIXME: Think through which of these functions should be inlineable and
which need to be callable, and change the code to match.

$Id: vxli.h,v 1.19 2006/02/21 16:16:59 hughett Exp $  $Name: pyvox-0-72 $
*****************************************************************************/


#ifndef VXLI_H
#define VXLI_H



/*-------------------------------------------------------------------------
Force prerequisite header files to be loaded
--------------------------------------------------------------------------*/

#ifndef CONFIG_H
#include "config.h"
#endif

#ifndef VOXEL_H
#include "voxel.h"
#endif



#ifdef __cplusplus          /* Allow to be called from C++ */
extern "C" {
#endif



/*-------------------------------------------------------------------------
Callable functions

Reserved for later use; currently, all internal functions are inlineable.
--------------------------------------------------------------------------*/

extern long vxli_find_kernel_coef(struct vxl_kernel *kern, long *delta);



/*-------------------------------------------------------------------------
Inlineable functions

These become inline functions on compilers that support that keyword
and ordinary extern functions otherwise.
--------------------------------------------------------------------------*/

inlineable long vxli_count(voxel_array *src);
inlineable long vxli_delta(voxel_array *src, long *coords);
inlineable long vxli_offset(voxel_array *src, long *coords);
inlineable long vxli_offset2(long rank, long *dimen, long *coords);
inlineable long vxli_step(voxel_array *src, int dim);
inlineable long vxli_step2(long rank, long *dimen, int dim);
inlineable unsigned char *vxli_locate(voxel_array *src, long *coords);
inlineable long vxli_offset_nocheck(voxel_array *src, long*coords);
inlineable int  vxli_same_shape   (voxel_array *src1, voxel_array *src2);
inlineable int  vxli_same_physical(voxel_array *src1, voxel_array *src2);
inlineable void vxli_default_physical(voxel_array *dest);
inlineable void vxli_copy_physical   (voxel_array *dest, voxel_array *src);
inlineable void vxli_set_physical    (voxel_array *dest, double *origin, 
    double *spacing);
inlineable void vxli_firstvoxel(int rank, long *coord);
inlineable int  vxli_nextvoxel (int rank, long *dimen, long *coord);
inlineable void vxli_firstrow  (int rank, long *coord);
inlineable int  vxli_nextrow   (int rank, long *dimen, long *coord);
inlineable void vxli_lastrow   (int rank, long *dimen, long *coord);
inlineable int  vxli_prevrow   (int rank, long *dimen, long *coord);
inlineable void vxli_firstrow2 (int rank, long *coord);
inlineable int  vxli_nextrow2  (int rank, long *dimen, int idim, long *coord);



/*-------------------------------------------------------------------------
Error messages for inlineable functions
-------------------------------------------------------------------------*/

extern const char vxli_errmsg_invalid  [];
extern const char vxli_errmsg_subscript[];
extern const char vxli_errmsg_badinfo  [];
extern const char vxli_errmsg_diffrank [];
extern const char vxli_errmsg_diffshape[];
extern const char vxli_errmsg_difforg  [];
extern const char vxli_errmsg_diffspace[];



/*-------------------------------------------------------------------------
Definitions of inlineable functions if compiler supports inline
--------------------------------------------------------------------------*/

#ifdef HAVE_INLINE
#include "vxli2.c"
#endif


#ifdef __cplusplus
}
#endif

#endif  /* VXLI_H */
