/**************************************************************************
decomment.c - Remove comments from a commented data file

Author:  Paul Hughett

Copyright (c) 1999 Trustees of the University of Pennsylvania
See the file COPYING for license terms.

This program removes the comments and blank lines from, and pastes
together the continuation lines of a commented data file, preparing
the file for SAS or another program that does not understand commented
data.  It takes its input from the file named on the command line, or
from standard input, and writes its output to standard output.

**************************************************************************/

static const char rcsid[] = "$Id: decomment.c,v 1.4 2005/12/29 18:33:14 hughett Exp $ Release $Name: pyvox-0-72 $";

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>

#include "cdata.h"
#include "memm.h"



/*------------------------------------------------------------------------
fatal - Report a fatal error condition

Generically speaking, this function is used to indicate an error
condition which makes it impossible for the calling function to
proceed.  This specific implementation treats such errors by printing
an error message and either aborting the program (and dumping core) or
terminating with a non-zero flag.  Other implementations, such as in a
command-driven program, might prefer to abort only the current command
but allow the user to continue with a new command.

FIXME:  This function is obsolete within Pyvox and is included directly
in this program rather than in a library.
-------------------------------------------------------------------------*/

static int errm_action = 2;   /* Action to take on error */

static void
fatal(
    const char *message,      /* Message as a format string */
    ...)                      /* Values as needed */
{
    va_list ap;

    va_start(ap, message);
    fprintf(stderr, "Error: ");
    vfprintf(stderr, message, ap);
    fprintf(stderr, "\n");
    va_end(ap);
    if (errm_action == 2)
	abort();
    else
	exit(EXIT_FAILURE);
}



/*------------------------------------------------------------------------
Main program
------------------------------------------------------------------------*/

int
main(int argc, char *argv[])
{
    FILE *input = stdin;      /* File to read, or standard input */
    int ndat;                 /* Number of data on current input line */
    char **dat;               /* Array of data fields */
    char **cmt;               /* Array of comment fields */
    int line;                 /* Line number in the input file */

    /* Open the input file; if none, use standard input */
    if (argc >= 2) {
	input = fopen(argv[1], "r");
	if (input == 0)
	    fatal("Unable to open input file %s", argv[1]);
    }

    /* Read each line, and write back out sans comments and other goodies */
    for (line = 1; !feof(input); line++) {
	ndat = cdata_read(input, &dat, &cmt);
	if (ferror(input))
	    fatal("Error reading input file %s", argv[1]);
	if (ndat > 0)
	    cdata_decomment(stdout, ndat, dat, cmt);
	cdata_free(ndat, dat, cmt);
    }

    exit(EXIT_SUCCESS);
}
