/****************************************************************************
vxli.c - Callable internal functions for the Voxel Kit

Author:  Paul Hughett

Copyright (c) 2001-2005 Trustees of the University of Pennsylvania
See the file COPYING for license terms.

This file is part of the Voxel Kit, a set of general-purpose functions
for volume image processing, with a bias toward medical imaging
applications.  It normally contains various functions for internal use
that are called rather than inlined; on a compiler that does not
support inlining, it also contains callable versions of the functions
that are normally inlined.

FIXME: Identify other common validity checking steps and isolate as
inlineable functions.
*****************************************************************************/

static const char rcsid[] = "$Id: vxli.c,v 1.11 2005/12/27 19:59:03 hughett Exp $ Release $Name: pyvox-0-72 $";

#include "config.h"

#include "errm.h"
#include "memm.h"
#include "voxel.h"
#include "vxli.h"



/*---------------------------------------------------------------------------
Compile inlineable functions as callable functions on unsupportive compilers
----------------------------------------------------------------------------*/

#ifndef HAVE_INLINE
#include "vxli2.c"
#endif



/*-------------------------------------------------------------------------
Error messages for inlineable functions

These error messages are defined here so that it is necessary to define
them only once, rather than each time that vxli2.c is included to define
the inlineable functions themselves.  Note that these symbols must also
be declared extern in the vxli.h header file.
---------------------------------------------------------------------------*/

const char vxli_errmsg_invalid[]   = "Invalid voxel array.";
const char vxli_errmsg_subscript[] = "Voxel array subscript out of range";
const char vxli_errmsg_badinfo[]   = "Invalid rank, dimen, or coords";
const char vxli_errmsg_diffrank[]  = "Arrays differ in rank";
const char vxli_errmsg_diffshape[] = "Arrays differ in shape";
const char vxli_errmsg_difforg[]   = "Arrays differ in physical origin";
const char vxli_errmsg_diffspace[] = "Arrays differ in physical spacing";
